import calendar
from datetime import datetime

import utils
from models.process_data import ProcessData
from models.process_result_data import ProcessResultData
from stores.data_store_interface import IDataStore


class DataSend:
    _error_arr = []
    _email_area_arr = []
    _email_area_virtual_arr = []

    @property
    def store(self) -> IDataStore:
        return self._store

    def __init__(self, store: IDataStore):
        self._store = store
        self._email_area_arr = self.store.query_all_emails()
        self._email_area_virtual_arr = self.store.query_all_virtual_emails()

    def send(self) -> None:
        self._error_arr = []
        list = self.store.query_to_send()
        utils.get_logger().info(f"开始发送邮件，数量为 {len(list)}")
        for item in list:
            self._send_item(item)
        if len(self._error_arr) > 0:
            self._send_email_no_found()

    def send_report_current_month(self):
        # 查询当月的数据
        start_date, end_date = self._get_first_and_last_day_of_current_month()
        self._send_reports(start_date, end_date)

    def send_report_prev_month(self):
        # 查询上月的数据
        start_date, end_date = self._get_first_and_last_day_of_prev_month()
        self._send_reports(start_date, end_date)

    def _send_reports(self, start_date, end_date):
        utils.get_logger().info(
            f"开始发送中标报告邮件，开始日期：{start_date.strftime("%Y-%m-%d")}，结束日期：{end_date.strftime("%Y-%m-%d")}"
        )
        email = self.store.query_master_email()
        if not email:
            utils.get_logger().error("没有找到master email")
            return
        items = self.store.query_to_report_by_date(start_date, end_date)
        title_prev = utils.get_config_value("email.report_title_prev", "【中标报告】")
        title = f"{start_date.month}月中标结果报告"
        body = self._build_report_email_html(title, items)
        flag = utils.send_email(email, f"{title_prev} {title}", body, True)
        if flag:
            utils.get_logger().info("发送中标报告邮件成功")

    def _send_item(self, item: ProcessData) -> None:
        utils.get_logger().info(f"开始发送邮件，地区为：{item.area} ，URL为 {item.url}")
        email = self._get_email_by_area(item.area)
        if not email:
            utils.get_logger().error(f"{item.area} 下没有找到email")
            if item.area not in self._error_arr:
                self._error_arr.append(item.area)
            return
        title_prev = utils.get_config_value("email.title_prev", "【招标信息】")
        body = self._build_email_html(item)
        flag = utils.send_email(
            email, f"{title_prev} {item.title}", body, True, item.attach_path
        )
        if flag:
            self.store.set_send(item.no)

    def _get_email_by_area(
        self, area: str, count: int = 0, virtual_area: str = None
    ) -> str:
        email = None
        area_str = area
        # if "省" in area:
        #     area_str_arr = area.split("省")
        #     area_str = area_str_arr[1] if len(area_str) > 1 else area_str_arr[0]
        # if "市" in area:
        #     area_str_arr = area.split("市")
        #     area_str = area_str_arr[0]
        for area_item in self._email_area_arr:
            if area_str in area_item.area:
                email = area_item.email
                if virtual_area:
                    new_area = f"{area_item.area},{virtual_area}"
                    self.store.update_area_email_area_by_name(area_item.name, new_area)
                    self._email_area_arr = self.store.query_all_emails()
                break
        if not email and count < 3:
            area_name = self._get_email_by_area_virtual(area_str)
            if area_name:
                virtual_area = (
                    f"{area_str},{virtual_area}" if virtual_area else area_str
                )
                email = self._get_email_by_area(area_name, count + 1, virtual_area)
        return email

    def _get_email_by_area_virtual(self, area: str) -> str:
        name = None
        for area_item in self._email_area_virtual_arr:
            if area in area_item.area:
                name = area_item.name
                break
        return name

    @staticmethod
    def _build_email_html(item: ProcessData, other: str = "") -> str:
        html_body = f"""
        <html>
        <head>
            <style>
                body {{
                    background-color: #f4f4f9;
                    font-family: Arial, sans-serif;
                    margin: 0;
                    padding: 20px;
                }}
                h1 {{
                    text-align: center;
                    color: #333;
                }}
                .container {{
                    max-width: 600px;
                    margin: 0 auto;
                    background-color: #fff;
                    padding: 20px;
                    border-radius: 8px;
                    box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);
                }}
                .button-container {{
                    text-align: center;
                    margin-top: 20px;
                }}
                .button {{
                    display: inline-block;
                    padding: 10px 20px;
                    font-size: 16px;
                    color: #fff!important;
                    background-color: #007bff;
                    text-decoration: none;
                    border-radius: 5px;
                    transition: background-color 0.3s;
                }}
                .button:hover {{
                    background-color: #0056b3;
                }}
                .system {{
                    color: #aaa;
                }}

            </style>
        </head>
        <body>
            <div class="container">
                <h1>{item.title}</h1>
                <p><strong>发布日期:</strong> {item.release_date}</p>
                <p><strong>招标编号:</strong> {item.no}</p>
                <p><strong>开标时间:</strong> {item.date}</p>
                <p><strong>开标地点:</strong> {item.address}</p>
                <p><strong>标书摘要:</strong> {item.summary}</p>
                <div class="button-container">
                    <a href="{item.url}" class="button">查看详情</a>
                </div>
                <div>
                    <h3>{other}</h3>
                </div>
                <p class="system">本邮件由系统自动发送，请勿回复。</p>

            </div>
        </body>
        </html>
        """
        return html_body

    def _build_report_email_html(self, title, items) -> str:
        body = ""
        for item in items:
            body += self._build_report_email_body(item)
        html = f"""
        <html>
        <head>
            <style>
                body {{
                    background-color: #f4f4f9;
                    font-family: Arial, sans-serif;
                    margin: 0;
                    padding: 20px;
                }}
                h1 {{
                    text-align: center;
                    color: #333;
                }}
                .container {{
                    max-width: 600px;
                    margin: 0 auto;
                    background-color: #fff;
                    padding: 20px;
                    border-radius: 8px;
                    box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);
                }}
                .button-container {{
                    text-align: center;
                    margin-top: 20px;
                }}
                .button {{
                    display: inline-block;
                    padding: 10px 20px;
                    font-size: 16px;
                    color: #fff!important;
                    background-color: #007bff;
                    text-decoration: none;
                    border-radius: 5px;
                    transition: background-color 0.3s;
                }}
                .button:hover {{
                    background-color: #0056b3;
                }}
                .system {{
                    color: #aaa;
                }}
                .card {{
                    background-color: #ffffff;
                    border: 1px solid #dddddd;
                    border-radius: 8px;
                    margin-bottom: 20px;
                    padding: 20px;
                    box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);
                }}
                .card h2 {{
                    margin-top: 0;
                }}
                .card p {{
                    margin: 0;
                }}
                .button-container {{
                    text-align: center;
                    margin-top: 15px;
                }}
                .button {{
                    display: inline-block;
                    padding: 6px 15px;
                    font-size: 14px;
                    color: #fff!important;
                    background-color: #007bff;
                    text-decoration: none;
                    border-radius: 3px;
                    transition: background-color 0.3s;
                }}
                .button:hover {{
                    background-color: #0056b3;
                }}
            </style>
        </head>
        <body>
            <div class="container">
                <h1>{title}</h1>
                {body}
                <p class="system">本邮件由系统自动发送，请勿回复。</p>
            </div>
        </body>
        </html>
        """
        return html

    @staticmethod
    def _build_report_email_body(item: ProcessResultData) -> str:
        body = f"""
           <div class="card">
               <h2>{item.title}</h2>
               <p><strong>项目编号:</strong> {item.no}</p>
               <p><strong>公告日期:</strong> {item.date}</p>
               <p><strong>关键词:</strong> {item.keyword}</p>
               <p><strong>价格:</strong> {item.price}</p>
               <p><strong>中标人:</strong> {item.bidder}</p>
               <p><strong>摘要:</strong> {item.summary}</p>
               <div class="button-container">
                <a href="{item.url}" class="button">查看详情</a>
               </div>
           </div>
           """
        return body

    def _send_email_no_found(self) -> None:
        email = utils.get_config_value("email.error_email")
        utils.get_logger().info(f"开始发送区域邮箱未匹配邮件: {email}")
        if not email:
            return
        title = "Warning: 相关地区没有匹配到邮箱，请及时添加相关配置"
        content = "以下区域中没有配置邮箱：\n\n    "
        content += "、".join(self._error_arr)
        content += "\n\n请及时添加相关配置。"
        utils.send_email(email, title, content, False, None)

    @staticmethod
    def _get_first_and_last_day_of_current_month():
        # 获取当前日期
        today = datetime.today()
        # 获取这个月的第一天
        first_day_of_current_month = datetime(today.year, today.month, 1, 0, 0, 0)
        # 获取这个月的最后一天
        _, last_day = calendar.monthrange(today.year, today.month)
        last_day_of_current_month = datetime(
            today.year, today.month, last_day, 23, 59, 59
        )
        return first_day_of_current_month, last_day_of_current_month

    @staticmethod
    def _get_first_and_last_day_of_prev_month():
        # 获取当前日期
        today = datetime.today()
        # 获取上个月的年份和月份
        if today.month == 1:
            prev_month_year = today.year - 1
            prev_month = 12
        else:
            prev_month_year = today.year
            prev_month = today.month - 1
        # 获取上个月的第一天
        first_day_prev_month = datetime(prev_month_year, prev_month, 1, 0, 0, 0)
        # 获取上个月的最后一天
        _, last_day = calendar.monthrange(prev_month_year, prev_month)
        last_day_of_prev_month = datetime(
            prev_month_year, prev_month, last_day, 23, 59, 59
        )
        return first_day_prev_month, last_day_of_prev_month
